import os
from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib import messages
from django.shortcuts import render
from django.shortcuts import render_to_response
from django.http import HttpResponse, Http404
from django.template import RequestContext
from django.core.files.storage import default_storage
from django.core.files.base import ContentFile
from django.core.urlresolvers import reverse_lazy, reverse
from django.views.generic import ListView
from django.views.generic.detail import DetailView
from django.views.generic.edit import (
    CreateView,
    UpdateView,
    DeleteView
)
from dal import autocomplete

from .utils import (
    listObjetivoHistoricoNacional, listObjetivoEstrategico,
    listObjetivoGenerales
)

from .models import (
    Proyecto, SubProyecto, Actividad,
    Alcance,
    CargaMasivaObjetivosHistoricos, CargaMasivaObjetivosNacional,
    CargaMasivaObjetivosEstrategico, CargaMasivaObjetivosGeneral
)

from utils.forms import (
    UploadFileForm
)

## Carga de datos del archivo csv a la base de datos
def carga_masiva_csv(ruta_archivo_hist, ruta_archivo_naci, ruta_archivo_estr, ruta_archivo_genr):
    n_reg_obj_hist = CargaMasivaObjetivosHistoricos.import_from_filename(ruta_archivo_hist)
    n_reg_obj_naci = CargaMasivaObjetivosNacional.import_from_filename(ruta_archivo_naci)
    n_reg_obj_estr = CargaMasivaObjetivosEstrategico.import_from_filename(ruta_archivo_estr)
    n_reg_obj_genr = CargaMasivaObjetivosGeneral.import_from_filename(ruta_archivo_genr)

    return n_reg_obj_hist, n_reg_obj_naci, n_reg_obj_estr, n_reg_obj_genr


def cargaObjetivos(request):
    """
        Funcion que permite cargar un archivo csv desde la interfez del coordinador recaudacion-FIDETEL
    """
    form = UploadFileForm()
    context = {'form': form}
    if request.method == 'POST':
        form = UploadFileForm(request.POST, request.FILES)
        if form.is_valid():
            archivo_objetivos = request.FILES['cargar']
            extension_objetivos = str(archivo_objetivos).split('.')
            longitud_historico = len(extension_objetivos)
            if extension_objetivos[longitud_historico - 1] == 'csv':
                data_historico = archivo_objetivos.readlines()
                formato_cabeceras = False
                mensaje_error = ""
                for cabecera_historico in range(0, 1):
                    separador_historico = data_historico[cabecera_historico].split(b"|")
                    if (separador_historico[0]).decode('UTF-8') == "OBJETIVOS HISTÓRICOS" \
                    and (separador_historico[1]).decode('UTF-8') == "OBJETIVOS NACIONALES" and \
                    (separador_historico[2]).decode('UTF-8') == "OBJETIVOS ESTRATÉGICOS" and \
                    (separador_historico[3]).decode('UTF-8') == "OBJETIVOS GENERALES\n":
                        formato_cabeceras = True
                    else:
                        formato_cabeceras = False
                        mensaje_error_historico = "Existe un error en la cabecera del csv, del formato del Plan de la Patria\n"
            else:
                mensaje_error = "El formato de archivo debe ser csv"
                messages.error(request, mensaje_error)
            if formato_cabeceras:
                datos_obj = listObjetivoHistoricoNacional(data_historico)
                datos_obj_est = listObjetivoEstrategico(data_historico, datos_obj[1])
                datos_obj_gene =listObjetivoGenerales(data_historico, datos_obj_est)
                archivo_obj_hist = ''.join(datos_obj[0])
                archivo_obj_naci = ''.join(datos_obj[1])
                archivo_obj_est = ''.join(datos_obj_est)
                archivo_obj_gene = ''.join(datos_obj_gene)
                print (archivo_obj_naci)
                path_historico = default_storage.save('tmp/formato_historico.csv', ContentFile(archivo_obj_hist))
                path_nacional = default_storage.save('tmp/formato_nacional.csv', ContentFile(archivo_obj_naci))
                path_estrategico = default_storage.save('tmp/formato_estrategico.csv', ContentFile(archivo_obj_est))
                path_general = default_storage.save('tmp/formato_general.csv', ContentFile(archivo_obj_gene))
                ruta_historico = os.path.join(settings.MEDIA_ROOT, path_historico)
                ruta_nacional = os.path.join(settings.MEDIA_ROOT, path_nacional)
                ruta_estrategico = os.path.join(settings.MEDIA_ROOT, path_estrategico)
                ruta_general = os.path.join(settings.MEDIA_ROOT, path_general)
                carga_masiva_csv(ruta_historico, ruta_nacional, ruta_estrategico, ruta_general)
                path_historico = default_storage.delete('tmp/formato_historico.csv')
                path_nacional = default_storage.delete('tmp/formato_nacional.csv')
                path_estrategico = default_storage.delete('tmp/formato_estrategico.csv')
                path_general = default_storage.delete('tmp/formato_general.csv')
                messages.success(request, 'La data se cargo con exito')
            else:
                messages.error(request, mensaje_error_historico)
        else:
            return render(request, 'utils/cargar_historica.html', context)

        context = {'form': form}
        return render(request, 'utils/cargar_historica.html', context)

    else:
        return render(request, 'utils/cargar_historica.html', context)


class AlcanceAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        # No se olvide de filtrar los resultados en función del visitante !
        if not self.request.user.is_authenticated():
            return Alcance.objects.none()

        qs = Alcance.objects.all()
        if self.q:
            qs = qs.filter(opcion_alcance__istartswith=self.q)

        return qs


class ProjectsList(ListView):
    model = Proyecto

    def __init__(self):
        super(ProjectsList, self).__init__()


class ProyectoCreation(LoginRequiredMixin, CreateView):
    model = Proyecto
    success_url = reverse_lazy('projects:consultar_proyectos')
    fields = [
                'fk_alcance', 'fk_sector', 'codigo', 'nombre',
                'justificacion', 'objetivo_general',
                'objetivos_especificos', 'monto_proyecto', 'fecha_inicio',
                'fecha_culminacion'
             ]


class ProyectoUpdate(UpdateView):
    model = Proyecto
    success_url = reverse_lazy('projects:consultar_proyectos')
    fields = [
                'fk_alcance', 'fk_sector', 'codigo', 'nombre',
                'justificacion', 'objetivo_general',
                'objetivos_especificos', 'monto_proyecto', 'fecha_inicio',
                'fecha_culminacion'
             ]


class SubProjectsList(ListView):
    model = SubProyecto

    def __init__(self):
        super(SubProjectsList, self).__init__()


class SubProyectoCreation(CreateView):
    model = SubProyecto
    success_url = reverse_lazy('projects:consultar_subproyectos')
    fields = [
                'fk_proyecto', 'parroquia', 'codigo', 'nombre',
                'descripcion_bien_servicio', 'unidad_medida',
                'meta', 'fecha_inicio', 'fecha_culminacion', 'comunidad',
                'monto_subproyecto'
             ]


class SubProyectoUpdate(UpdateView):
    model = SubProyecto
    success_url = reverse_lazy('projects:consultar_subproyectos')
    fields = [
                'fk_proyecto', 'parroquia', 'codigo', 'nombre',
                'descripcion_bien_servicio', 'unidad_medida',
                'meta', 'fecha_inicio', 'fecha_culminacion', 'comunidad',
                'monto_subproyecto'
             ]


class ActividadList(ListView):
    model = Actividad

    def __init__(self):
        super(ActividadList, self).__init__()


class ActividadCreation(CreateView):
    model = Actividad
    success_url = reverse_lazy('projects:consultar_actividades')
    fields = [
                'fk_subproyecto', 'descripcion', 'unidad_medida', 'fecha_inicio',
                'fecha_culminacion', 'horas_estimadas_ejecucion',
                'costo_estimadao_ejecucion', 'peso_actividad',
                'cantidad_estimada_trabajo'
             ]


class ActividadUpdate(UpdateView):
    success_url = reverse_lazy('projects:consultar_actividades')
    fields = [
                'fk_subproyecto', 'descripcion', 'unidad_medida', 'fecha_inicio',
                'fecha_culminacion', 'horas_estimadas_ejecucion',
                'costo_estimadao_ejecucion', 'peso_actividad',
                'cantidad_estimada_trabajo'
             ]